function [ptsCMap,RCs,ptsBad,RCsBad] = BuildScaleConvergenceMap(HalfWindowSize,Img,theVoteAreaRatio,thEigenvalueRatio,thresholdDeterminant,thresholdRMS)
% This file is part of ASN operator inspired from the article:
% bibtex
% @inproceedings{conf/eccv/OuelletH08,
% title = {ASN: Image Keypoint Detection from Adaptive Shape Neighborhood.},
% author = {Jean-Nicolas Ouellet and Patrick Hbert},
% booktitle = {ECCV (1)},
% crossref = {conf/eccv/2008-1},
% editor = {David A. Forsyth and Philip H. S. Torr and Andrew Zisserman},
% pages = {454-467},
% publisher = {Springer},
% series = {Lecture Notes in Computer Science},
% url = {http://dblp.uni-trier.de/db/conf/eccv/eccv2008-1.html#OuelletH08},
% volume = {5302},
% year = {2008},
% description = {dblp},
% date = {2008-10-20}, ee = {http://dx.doi.org/10.1007/978-3-540-88682-2_35}, isbn = {978-3-540-88681-5},
% keywords = {dblp }
% }
% 
%     ASN operator is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ASN operator is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
% 
% Jean-Nicolas Ouellet 2008
% Laval university
% jouellet@gel.ulaval.ca


ptsBad = [];
RCsBad = [];
RCs = [];
ptsCMap = [];

showWaitbar=0;
showResults = 0;
ptsSMap = [];
ptsCMap = [];

if ~exist('theVoteAreaRatio')
    disp('ASN - Default parameters')
    theVoteAreaRatio = [0.15 0.1 0.1];
    thEigenvalueRatio = 0.1;
    thresholdDeterminant = 1e-6;
    thresholdRMS = 0.25;
end

thresholdVoteDistanceFromPole = 1;

sImg = size(Img(:,:,1));
sumMap = zeros(sImg);
convergenceMap = zeros(sImg);
SolutionX = zeros(sImg);
SolutionY = zeros(sImg);


RMS = zeros(sImg);
sWindow=1;

voteMethod=1;

sigma = 1;
[G,dG] = GetDerivativeFilter(sigma);
opx=G'*dG;
opy=dG'*G;


for i=1:size(Img,3)
    Dx{i} = imfilter(Img(:,:,i),opx,'replicate');
    Dy{i} = imfilter(Img(:,:,i),opy,'replicate');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Facultative step !
%reweight gradient with tensor eigenvalue ratio to reduce places where
%more than 1 edge are present in the derivation kernel. Prevents bad gradients from affecting
%operator
% convolutionMaskGradient = G'*G;
% Dx2 = zeros(sImg);
% Dy2 = zeros(sImg);
% DxDy = zeros(sImg);
% for channel=1:size(Img,3)
%     Dx2 = Dx2+double(imfilter(Dx{channel}.*Dx{channel},convolutionMaskGradient,'same'));
%     Dy2 = Dy2+double(imfilter(Dy{channel}.*Dy{channel},convolutionMaskGradient,'same'));
%     DxDy = DxDy+double(imfilter(Dx{channel}.*Dy{channel},convolutionMaskGradient,'same'));
% end
% 
% a = Dx2;
% c = Dy2;
% b = DxDy;
% eig1 = (1/2*a+1/2*c+1/2*(a.^2-2*a.*c+c.^2+4*b.^2).^(1/2));
% eig2 = (1/2*a+1/2*c-1/2*(a.^2-2*a.*c+c.^2+4*b.^2).^(1/2));
% ok = eig2>10^-12;
% ratio = zeros(size(eig1));
% ratio(ok) = abs(eig1(ok)./eig2(ok));
% weight = 1-exp(1-ratio.^.5);
% disp('weight invalid gradient enabled')
% for channel=1:size(Img,3)
%     Dx{channel} = Dx{channel}.*weight;
%     Dy{channel} = Dy{channel}.*weight;
% end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%generate indexes of positions
[X,Y] = meshgrid(1:sImg(2),1:sImg(1));
Ind = ConvertXYtoInd(X,Y,sImg);
[voteBaseX,voteBaseY] = meshgrid(-2:2,-2:2);
voteBaseInd = voteBaseX*sImg(1)+voteBaseY;
maskSecureZone = voteBaseInd(:);


HalfWindowSizeBase = HalfWindowSize;
voteMask = logical(zeros(sImg));
indRC = 1;
indRCBad=1;
Rejected = [];

for scale=3:-1:1
    HalfWindowSize = HalfWindowSizeBase*scale;
    thresholdVoteMinimum = theVoteAreaRatio(scale)*(pi*HalfWindowSize^2);
    
    %generate circular integration kernel with uniform weights
    [XBase,YBase] = meshgrid(-HalfWindowSize:HalfWindowSize,-HalfWindowSize:HalfWindowSize);
    rr = sqrt(XBase.^2+YBase.^2);
    ok = rr<=HalfWindowSize;
    convolutionMask = zeros(HalfWindowSize*2+1);
    convolutionMask(ok) = 1;
    convolutionMask = convolutionMask./sum(convolutionMask(:));
    

    Dx2 = zeros(sImg);
    Dy2 = zeros(sImg);
    DxDy = zeros(sImg);
    DxDyY = zeros(sImg);
    DxDyX = zeros(sImg);
    Dx2X = zeros(sImg);
    Dy2Y = zeros(sImg);
    for channel=1:size(Img,3)
        Dx2 = Dx2+imfilter(Dx{channel}.*Dx{channel},convolutionMask,'same');
        Dy2 = Dy2+imfilter(Dy{channel}.*Dy{channel},convolutionMask,'same');
        DxDy = DxDy+imfilter(Dx{channel}.*Dy{channel},convolutionMask,'same');
        DxDyY = DxDyY+imfilter(Y.*Dx{channel}.*Dy{channel},convolutionMask,'same');
        DxDyX = DxDyX+imfilter(X.*Dx{channel}.*Dy{channel},convolutionMask,'same');
        Dx2X = Dx2X+imfilter(Dx{channel}.*Dx{channel}.*X,convolutionMask,'same');
        Dy2Y = Dy2Y+imfilter(Dy{channel}.*Dy{channel}.*Y,convolutionMask,'same');
    end


    %prevalidation of solution at each position in the image
    
    %stong and valid solution
    DetA = Dx2.*Dy2-DxDy.^2;
    %solution inside the image
    okPosition = X>HalfWindowSize+1 & X<sImg(2)-HalfWindowSize-1 & Y>HalfWindowSize+1 & Y<sImg(1)-HalfWindowSize-1;
    %solution not on edge
    r = 1/thEigenvalueRatio;
    S1 = 1/2*Dy2+1/2*Dx2+1/2*(Dy2.^2-2*Dx2.*Dy2+Dx2.^2+4*DxDy.^2).^(1/2);
    S2 = 1/2*Dy2+1/2*Dx2-1/2*(Dy2.^2-2*Dx2.*Dy2+Dx2.^2+4*DxDy.^2).^(1/2);
    nodivzero = S2>10^-10;
    th = zeros(size(S1));
    th(nodivzero) = S1(nodivzero)./S2(nodivzero)>1/r & S1(nodivzero)./S2(nodivzero)<r;
    detOk = DetA(:)>thresholdDeterminant & th(:) & okPosition(:);
    
    

    from = ConvertXYtoInd(X(detOk),Y(detOk),sImg);
    localSolutionX = [(Dy2(detOk).*(Dx2X(detOk)+DxDyY(detOk))-DxDy(detOk).*(Dy2Y(detOk)+DxDyX(detOk)))./(Dx2(detOk).*Dy2(detOk)-DxDy(detOk).^2)];
    localSolutionY = [(Dx2(detOk).*(Dy2Y(detOk)+DxDyX(detOk))-DxDy(detOk).*(Dx2X(detOk)+DxDyY(detOk)))./(Dx2(detOk).*Dy2(detOk)-DxDy(detOk).^2)];
    SolutionX(from) =  localSolutionX;
    SolutionY(from) =  localSolutionY;

    Xto = round(localSolutionX);
    Yto = round(localSolutionY);
    Xfrom = X(detOk);
    Yfrom = Y(detOk);
    
    
    %invalidate
    v = [localSolutionX(:) localSolutionY(:)]-[Xfrom Yfrom];
    vn = sqrt(v(:,1).^2+v(:,2).^2);
    
    
    %keep solution voting near estimation window
    v = [localSolutionX(:) localSolutionY(:)]-[Xfrom Yfrom];
    vn = sqrt(v(:,1).^2+v(:,2).^2);
    ok = Xto>HalfWindowSize+1 & Xto<sImg(2)-HalfWindowSize-1 & Yto>HalfWindowSize+1 & Yto<sImg(1)-HalfWindowSize-1 &  vn<1.5*HalfWindowSize;


    Xto = Xto(ok);
    Yto = Yto(ok);
    localSolutionX = localSolutionX(ok);
    localSolutionY = localSolutionY(ok);
    Xfrom = Xfrom(ok);
    Yfrom = Yfrom(ok);


    from = ConvertXYtoInd(Xfrom,Yfrom,sImg);
    to = ConvertXYtoInd(Xto,Yto,sImg);


    convergenceMap(from) = to;

    % donne un vote dpendant de la distance du pt subpixel de
    % l'accumulateur.
    % p1--------p2
    %  |----X----|
    %  |    |    |
    % p3--------p4


    %p1
    ptx = floor(localSolutionX);
    pty = floor(localSolutionY);

    px = localSolutionX-ptx;
    py = localSolutionY-pty;

    ind1 = ((ptx(:)-1)*sImg(1)+pty(:));
    ind2 = ind1+1;
    ind3 = ind1+sImg(1);
    ind4 = ind1+sImg(1)+1;

    %equivalent to next loop but faster
    sum1 = accumarray(ind1(:), (1-px(i)).*(1-py(i))./sWindow,[numel(sumMap) 1]);
    sum2 = accumarray(ind2(:), (1-px(i)).*   py(i) ./sWindow,[numel(sumMap) 1]);
    sum3 = accumarray(ind3(:),    px(i).* (1-py(i))./sWindow,[numel(sumMap) 1]);
    sum4 = accumarray(ind4(:),    px(i).*    py(i) ./sWindow,[numel(sumMap) 1]);
    sumMap = reshape(sum1+sum2+sum3+sum4,size(sumMap));

    

%         for i=1:length(ind1)
% 
%             if mod(i,1000)==0
%                 if showWaitbar==1
% %                 waitbar( (4-scale)*i/(3*length(ptx)),bb);
%                 end
%             end
%             sumMap(ind1(i)) = sumMap(ind1(i))+  (1-px(i)).*(1-py(i))./sWindow;
%             sumMap(ind2(i)) = sumMap(ind2(i))+  (1-px(i)).*   py(i) ./sWindow;
%             sumMap(ind3(i)) = sumMap(ind3(i))+     px(i).* (1-py(i))./sWindow;
%             sumMap(ind4(i)) = sumMap(ind4(i))+     px(i).*    py(i) ./sWindow;
%         end
    

    sumMap(:,1:4) = 0;
    sumMap(1:4,:) = 0;
    sumMap(:,end-3:end) = 0;
    sumMap(end-3:end,:) = 0;
    
    %%Local maxima extraction
    indSMap = findLocalMaxima(sumMap);
    ok = voteMask(indSMap)==0;    
    indSMap = indSMap(ok);
    valSMap = sumMap(indSMap);
    
    %gives priority to strong poles vs weak poles
    valVotes = valSMap;
    [valVotes,IndSorted] = sort(valVotes,'descend');
    valSMap = valSMap(IndSorted);
    indSMap = indSMap(IndSorted);
    
    
    %%Validation with minimal vote (remove noisy poles)
    th = valVotes>thresholdVoteMinimum;
    valSMap = valSMap(th);
    indSMap = indSMap(th);
    valVotes = valVotes(th);
    
    inside = indSMap-sImg(1) <1 | indSMap+sImg(1)>sImg(2) | indSMap-1 <1 & indSMap+1>sImg(1);
    valSMap = valSMap(inside);
    indSMap = indSMap(inside);
    valVotes = valVotes(inside);
    
    %this pre-find will accelerate search for support map
    indCMap = find(convergenceMap);
    valCMap = convergenceMap(indCMap);

    %extract coordinates
    indToRemove = [];
    reasonToRemove = [];    
    for mm=1:length(indSMap)
        
        %already a pole detected at this position
        if  voteMask(indSMap(mm))==1
            indToRemove = [indToRemove;mm];
            continue;
        end
        
        %subpixel position of detected maximum
        ptSMap = LocalisationSMap(sumMap,indSMap(mm));
        %extraction of position voting for this pole
        extractedIndCMap = ExtractionCMap(indSMap(mm),indCMap,valCMap,sImg);
        poles = [SolutionX(extractedIndCMap) SolutionY(extractedIndCMap)];
        radiusFromPole = sqrt( (poles(:,1)-ptSMap(1)).^2 + (poles(:,2)-ptSMap(2)).^2);
        ok = radiusFromPole<thresholdVoteDistanceFromPole;
        %support map
        extractedIndCMapTemp = extractedIndCMap(ok);
        
        %extended support map
        extractedIndCMapTempDilated = DilateRegion(sImg,extractedIndCMapTemp,HalfWindowSize);
        
        
        %estimate uncertainty of pole from extended support map
        localDx = [];
        localDy = [];
        for channel=1:size(Img,3)
            localDx = [localDx;Dx{channel}(extractedIndCMapTempDilated)];
            localDy = [localDy;Dy{channel}(extractedIndCMapTempDilated)];
        end
        Xloc = repmat(X(extractedIndCMapTempDilated),size(Img,3),1);
        Yloc = repmat(Y(extractedIndCMapTempDilated),size(Img,3),1);
        poleWeight = GetGaussianValue(radiusFromPole,0.5);
        ptCMap = [sum(poles(:,1).*poleWeight)/sum(poleWeight) sum(poles(:,2).*poleWeight)/sum(poleWeight)];
        ptSub = LocalisationCMap(localDx,localDy,Xloc,Yloc);
        ptCMap = [ptCMap(1:2) ptSub(3:end) DetA(indSMap(mm))];
        
        
        if ptSub(4)/ptSub(3)<thEigenvalueRatio | ptSub(end-1)>thresholdRMS
            indToRemove = [indToRemove;mm];
            reasonToRemove = [reasonToRemove;0 ptSub(end)>thresholdRMS ptSub(4)/ptSub(3)<thEigenvalueRatio];
            continue
        end
        RCs(indRC).RC = [extractedIndCMapTemp Dx{1}(extractedIndCMapTemp) Dy{1}(extractedIndCMapTemp) X(extractedIndCMapTemp) Y(extractedIndCMapTemp)];
        RCs(indRC).RCplus = [extractedIndCMapTempDilated  Dx{1}(extractedIndCMapTempDilated) Dy{1}(extractedIndCMapTempDilated) X(extractedIndCMapTempDilated) Y(extractedIndCMapTempDilated)];
        ptsCMap = [ptsCMap;[ptCMap length(extractedIndCMapTemp) scale]];
        indRC = indRC+1;
        
        for vmaskind = 1:length(maskSecureZone)        
            voteMask(indSMap(mm)+maskSecureZone(vmaskind))=1;
        end
    end
    


    %preparing for next scale
    indSMap(indToRemove) = [];
    sumMap = sumMap*0;
    SolutionX = SolutionX.*0;
    SolutionY = SolutionY.*0;
    convergenceMap = convergenceMap.*0;
    
end
