function Unc = EvaluteSolutionUncertainty(AA,BB,Dx,X,Dy,Y,ptSub)
%propagate uncertainty of fitting to estimated paramters
%Unc = [ Varx Vary EllipseAxis1 RMS varianceResiduals]

% This file is part of ASN operator inspired from the article:
% bibtex
% @inproceedings{conf/eccv/OuelletH08,
% title = {ASN: Image Keypoint Detection from Adaptive Shape Neighborhood.},
% author = {Jean-Nicolas Ouellet and Patrick Hbert},
% booktitle = {ECCV (1)},
% crossref = {conf/eccv/2008-1},
% editor = {David A. Forsyth and Philip H. S. Torr and Andrew Zisserman},
% pages = {454-467},
% publisher = {Springer},
% series = {Lecture Notes in Computer Science},
% url = {http://dblp.uni-trier.de/db/conf/eccv/eccv2008-1.html#OuelletH08},
% volume = {5302},
% year = {2008},
% description = {dblp},
% date = {2008-10-20}, ee = {http://dx.doi.org/10.1007/978-3-540-88682-2_35}, isbn = {978-3-540-88681-5},
% keywords = {dblp }
% }
% 
%     ASN operator is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ASN operator is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
% 
% Jean-Nicolas Ouellet 2008
% Laval university
% jouellet@gel.ulaval.ca

ptSub = ptSub(:)';

%fitting uncertainty
% s = ptSub';
% B = -Dx.*X-Y.*Dy;
% BTB = (B'*B);
% R = (s'*AA*s-2*s'*BB+BTB)/(length(Dx)-2);
% cvar2_constantVariance = R*inv(AA);
% [U,S,V] = svd(cvar2_constantVariance);
% S = diag(S)';
K = [Dx Dy];
Kf = K*ptSub(1:2)';
b = Dx.*X+Dy.*Y;
A = K'*K;
sig2 = norm(Kf-b).^2/(length(Dx)-2);
covariance = sig2*inv(A);
[U,S,V] = svd(covariance);



%fitting rms error
Errnorm = abs(Dx*ptSub(1) + Dy.*ptSub(2) - (Dx.*X + Dy.*Y));
RMS = mean(Errnorm);


Unc = [diag(S)' V(:,1)' RMS sig2];