function ptSub = LocalisationCMap(localDx,localDy,localXs,localYs)
%point de convergence du gradient de convergence map
% This file is part of ASN operator inspired from the article:
% bibtex
% @inproceedings{conf/eccv/OuelletH08,
% title = {ASN: Image Keypoint Detection from Adaptive Shape Neighborhood.},
% author = {Jean-Nicolas Ouellet and Patrick Hbert},
% booktitle = {ECCV (1)},
% crossref = {conf/eccv/2008-1},
% editor = {David A. Forsyth and Philip H. S. Torr and Andrew Zisserman},
% pages = {454-467},
% publisher = {Springer},
% series = {Lecture Notes in Computer Science},
% url = {http://dblp.uni-trier.de/db/conf/eccv/eccv2008-1.html#OuelletH08},
% volume = {5302},
% year = {2008},
% description = {dblp},
% date = {2008-10-20}, ee = {http://dx.doi.org/10.1007/978-3-540-88682-2_35}, isbn = {978-3-540-88681-5},
% keywords = {dblp }
% }
% 
%     ASN operator is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ASN operator is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
% 
% Jean-Nicolas Ouellet 2008
% Laval university
% jouellet@gel.ulaval.ca

AA =      [ sum(localDx.^2)            sum(localDy.*localDx)
            sum(localDy.*localDx)      sum(localDy.^2) ];
BB =      [ sum(localDx.^2.*localXs) + sum(localDx.*localDy.*localYs)
            sum(localDy.^2.*localYs) + sum(localDx.*localDy.*localXs)];


DetA = AA(1,1)*AA(2,2)-AA(1,2)^2;
if abs(DetA)<1e-16
    %matrix not inversible:unstable solution
    ptSub = [-1 -1 Inf];
    return
end
ptSub = (AA\BB)';
Unc = EvaluteSolutionUncertainty(AA,BB,localDx,localXs,localDy,localYs,ptSub);
ptSub = [ptSub Unc];



